'use strict';

var Redacted = require('effect/Redacted');
var S = require('effect/Schema');
var predicates_js = require('./predicates.cjs');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var Redacted__namespace = /*#__PURE__*/_interopNamespace(Redacted);
var S__namespace = /*#__PURE__*/_interopNamespace(S);

const ValidVersion = S__namespace.Literal(
  "2024-01",
  "2024-04",
  "2024-07",
  "2024-10",
  "2025-01",
  "unstable"
).annotations({
  identifier: "ValidVersion",
  title: "Valid Version",
  description: "Valid version for the Storefront client",
  examples: ["2024-01", "2024-04", "2024-07", "2024-10", "unstable"]
});
const LatestVersion = ValidVersion.pipe(
  S__namespace.pickLiteral("2024-10")
).annotations({
  identifier: "LatestVersion",
  title: "Latest Version",
  description: "Latest version for the Storefront client",
  examples: ["2024-10"]
});
const ContentType = S__namespace.Literal("json", "graphql").annotations({
  identifier: "ContentType",
  title: "Content Type",
  description: "Content type for the Storefront client",
  examples: ["json", "graphql"]
});
const BaseClientOptions = S__namespace.Struct({
  apiVersion: S__namespace.optionalWith(ValidVersion, {
    default: () => LatestVersion.literals[0]
  }),
  storeName: S__namespace.NonEmptyString,
  retries: S__namespace.optionalWith(S__namespace.Int, { default: () => 2 }),
  contentType: S__namespace.optionalWith(ContentType, { default: () => "json" })
}).annotations({
  identifier: "BaseClientOptions",
  title: "Base Client Options",
  description: "Base client options for the Storefront client",
  examples: [
    {
      apiVersion: "2024-10",
      retries: 1,
      storeName: "solidifront",
      contentType: "json"
    }
  ]
});
const PrivateToken = S__namespace.Redacted(
  S__namespace.NonEmptyString.pipe(
    S__namespace.startsWith("shpat_", {
      message: () => "Invalid private access token format, expected format: shpat_************"
    }),
    S__namespace.filter(
      () => predicates_js.isServer() || "private access tokens and headers should only be used in a server-to-server implementation. Use the public API access token in nonserver environments."
    )
  )
).annotations({
  identifier: "PrivateToken",
  title: "Private Token",
  description: "Private API access token for the Storefront client",
  examples: [Redacted__namespace.make("shpat_****************************")],
  redacted: Redacted__namespace.make("shpat_****************************"),
  sensitive: true
});
const PublicToken = S__namespace.NonEmptyString.annotations({
  identifier: "PublicToken",
  title: "Public Token",
  description: "Public API access token for the Storefront client",
  examples: ["****************************"]
});
const TokenFields = S__namespace.Struct({
  privateAccessToken: S__namespace.optional(PrivateToken),
  publicAccessToken: S__namespace.optional(PublicToken)
}).pipe(
  S__namespace.filter((fields) => {
    if (predicates_js.hasPublicAccessToken(fields) && predicates_js.hasPrivateAccessToken(fields))
      return "only provide either a public or private access token";
    if (!predicates_js.hasPublicAccessToken(fields) && !predicates_js.hasPrivateAccessToken(fields))
      return "a public or private access token must be provided";
    return true;
  }),
  S__namespace.annotations({
    identifier: "TokenFields",
    title: "Token Fields",
    description: "Token fields for the Storefront API",
    examples: [
      {
        privateAccessToken: Redacted__namespace.make(
          "shpat_********************************"
        )
      },
      {
        publicAccessToken: "********************************"
      }
    ]
  })
);
const ClientOptions = S__namespace.asSchema(
  S__namespace.extend(BaseClientOptions, TokenFields)
).annotations({
  identifier: "ClientOptions",
  title: "Client Options",
  description: "Options for initializing the a Storefront client",
  examples: [
    {
      apiVersion: "2024-10",
      storeName: "solidifront",
      retries: 2,
      publicAccessToken: "********************************",
      contentType: "json"
    },
    {
      apiVersion: "2024-10",
      storeName: "solidifront",
      retries: 2,
      privateAccessToken: Redacted__namespace.make(
        "shpat_********************************"
      ),
      contentType: "graphql"
    }
  ],
  parseOptions: {
    onExcessProperty: "error"
  }
});
const GraphQLJsonBody = S__namespace.Struct({
  data: S__namespace.optional(S__namespace.Unknown),
  extensions: S__namespace.optional(
    S__namespace.Record({
      key: S__namespace.String,
      value: S__namespace.Any
    })
  ),
  errors: S__namespace.optional(
    S__namespace.Array(
      S__namespace.Struct({
        message: S__namespace.NonEmptyString,
        locations: S__namespace.optional(
          S__namespace.Array(S__namespace.Struct({ line: S__namespace.Int, column: S__namespace.Int }))
        ),
        extensions: S__namespace.optional(
          S__namespace.Struct({
            code: S__namespace.NonEmptyString
          })
        )
      })
    )
  )
}).annotations({
  identifier: "GraphQLJsonBody",
  title: "GraphQL JSON Body",
  description: "The GraphQL JSON body for the Storefront API",
  examples: [
    {
      errors: [
        {
          message: "Throttled",
          extensions: {
            code: "THROTTLED"
          }
        }
      ],
      extensions: {}
    },
    {
      data: {
        shop: {
          name: "Solidifront"
        }
      },
      extensions: {}
    }
  ]
});
const RequestOptions = S__namespace.partial(
  S__namespace.Struct({
    apiVersion: ValidVersion,
    storeName: S__namespace.NonEmptyString,
    contentType: ContentType,
    buyerIp: S__namespace.NonEmptyString,
    variables: S__namespace.Record({
      key: S__namespace.String,
      value: S__namespace.Any
    }),
    privateAccessToken: PrivateToken,
    publicAccessToken: S__namespace.NonEmptyString,
    signal: S__namespace.optional(S__namespace.instanceOf(AbortSignal))
  })
).annotations({
  identifier: "RequestOptions",
  title: "Request Options",
  description: "Options for making requests to the Storefront API",
  examples: [
    {
      apiVersion: "2024-01"
    },
    {
      storeName: "my-store"
    },
    {
      variables: {
        productID: "12345"
      },
      publicAccessToken: "********************************"
    },
    {
      storeName: "my-store",
      variables: {
        productID: "12345"
      },
      buyerIp: "127.0.0.1",
      privateAccessToken: Redacted__namespace.make(
        "shpat_********************************"
      )
    }
  ],
  parseOptions: {
    onExcessProperty: "error"
  }
});

exports.ClientOptions = ClientOptions;
exports.ContentType = ContentType;
exports.GraphQLJsonBody = GraphQLJsonBody;
exports.LatestVersion = LatestVersion;
exports.RequestOptions = RequestOptions;
exports.ValidVersion = ValidVersion;
//# sourceMappingURL=schemas.cjs.map
//# sourceMappingURL=schemas.cjs.map