import * as Effect from 'effect/Effect';
import * as TypedStorefrontClient from './services/TypedStorefrontClient.js';
import * as Logger from 'effect/Logger';
import * as LogLevel from 'effect/LogLevel';
import * as Layer from 'effect/Layer';

const createStorefrontClient = ({
  logger,
  ...initOptions
}) => {
  let layer = TypedStorefrontClient.Default;
  let minimumLogLevel = LogLevel.None;
  let loggerLayer = Logger.pretty;
  if (logger) {
    loggerLayer = Logger.replace(Logger.defaultLogger, Logger.make(logger));
    layer = Layer.mergeAll(loggerLayer, layer);
    minimumLogLevel = LogLevel.All;
  }
  return Effect.runSync(
    Effect.gen(function* () {
      const client = yield* TypedStorefrontClient.make(initOptions);
      return {
        mutate: (mutation, options) => Effect.runPromise(
          client.mutate(mutation, options).pipe(
            Logger.withMinimumLogLevel(minimumLogLevel),
            Effect.provide(loggerLayer),
            Effect.catchAll((error) => {
              if (error._tag === "ExtractOperationNameError" || error._tag === "ParseError" || error._tag === "RequestError" || error._tag === "ResponseError") {
                return Effect.fail(
                  new Error(error.message, {
                    cause: error.cause
                  })
                );
              }
              if (error._tag === "HttpBodyError") {
                const reason = error.reason;
                if (reason._tag === "JsonError") {
                  return Effect.fail(
                    new Error("Failed to parse JSON response")
                  );
                }
                if (reason._tag === "SchemaError") {
                  return Effect.fail(
                    new Error(reason.error.message, {
                      cause: reason.error.cause
                    })
                  );
                }
              }
              return Effect.fail(new Error("Something went horribly wrong!"));
            })
          ),
          {
            signal: options?.signal
          }
        ),
        query: (query, options) => Effect.runPromise(
          client.query(query, options).pipe(
            Logger.withMinimumLogLevel(minimumLogLevel),
            Effect.provide(loggerLayer),
            Effect.catchAll((error) => {
              if (error._tag === "ExtractOperationNameError" || error._tag === "ParseError" || error._tag === "RequestError" || error._tag === "ResponseError") {
                return Effect.fail(
                  new Error(error.message, {
                    cause: error.cause
                  })
                );
              }
              if (error._tag === "HttpBodyError") {
                const reason = error.reason;
                if (reason._tag === "JsonError") {
                  return Effect.fail(
                    new Error("Failed to parse JSON response")
                  );
                }
                if (reason._tag === "SchemaError") {
                  return Effect.fail(
                    new Error(reason.error.message, {
                      cause: reason.error.cause
                    })
                  );
                }
              }
              return Effect.fail(new Error("Something went horribly wrong!"));
            })
          ),
          {
            signal: options?.signal
          }
        )
      };
    }).pipe(Logger.withMinimumLogLevel(minimumLogLevel), Effect.provide(layer))
  );
};

export { createStorefrontClient };
//# sourceMappingURL=index.js.map
//# sourceMappingURL=index.js.map