import * as Effect from 'effect/Effect';
import * as LoggerUtils from './LoggerUtils.js';
import * as LogLevel from 'effect/LogLevel';
import { isQuery, isMutation } from '../predicates.js';
import { ExtractOperationNameError, AssertQueryError, AssertMutationError } from '../errors.js';

const minify = (operation) => Effect.succeed(
  operation.replace(/\s*#.*$/gm, "").replace(/\s+/gm, " ").trim()
);
const extractName = (operation) => Effect.succeed(operation).pipe(
  Effect.filterOrFail(
    (operation2) => {
      const match = operation2.match(/\b(query|mutation)\b\s+(\w+)/);
      return !!match;
    },
    (operation2) => new ExtractOperationNameError(operation2)
  ),
  Effect.andThen((operation2) => {
    const match = operation2.match(/\b(query|mutation)\b\s+(\w+)/);
    return Effect.succeed(match[2]);
  })
);
const assert = ({
  type,
  operation
}) => Effect.succeed(operation).pipe(
  Effect.filterOrDie(
    (o) => type === "query" ? isQuery(o) : isMutation(o),
    (o) => type === "query" ? new AssertQueryError(o) : new AssertMutationError(o)
  )
);
const annotate = ({
  type,
  operation: o,
  variables
}) => LoggerUtils.filterLevelOrNever(
  LogLevel.None,
  Effect.gen(function* () {
    const name = yield* extractName(o);
    return yield* Effect.annotateLogsScoped({
      name,
      type,
      operation: o,
      variables
    });
  })
);
const validate = ({
  type,
  operation,
  variables
}) => Effect.gen(function* () {
  const minifiedOperation = yield* minify(operation);
  yield* assert({ type, operation: minifiedOperation });
  yield* annotate({
    type,
    operation: minifiedOperation,
    variables
  });
  return minifiedOperation;
});

export { annotate, assert, extractName, minify, validate };
//# sourceMappingURL=StorefrontOperation.js.map
//# sourceMappingURL=StorefrontOperation.js.map